@@ -52,7 +52,7 @@
 #elif defined (__QNXNTO__) || defined (__PLAYBOOK__)
 #   define  TARGET_HOST_BLACKBERRY  1
 
-#elif defined(__posix__) || defined(__unix__) || defined(__linux__) || defined(__sun)
+#elif defined(__posix__) || defined(unix) || defined(__unix__) || defined(__linux__) || defined(__sun)
 #   if defined(FREEGLUT_WAYLAND)
 #      define  TARGET_HOST_POSIX_WAYLAND  1
 #   else
@@ -64,6 +64,9 @@
 #   define  TARGET_HOST_POSIX_X11  1
 /* #   define  TARGET_HOST_MAC_OSX    1 */
 
+#elif defined(__wii__) || defined(__gamecube__)
+#   define TARGET_HOST_OGC  1
+
 #else
 #   error "Unrecognized target host!"
 
@@ -103,6 +106,10 @@
 #   define  TARGET_HOST_SOLARIS        0
 #endif
 
+#ifndef  TARGET_HOST_OGC
+#   define  TARGET_HOST_OGC            0
+#endif
+
 /* -- FIXED CONFIGURATION LIMITS ------------------------------------------- */
 
 #define  FREEGLUT_MAX_MENUS            3
@@ -210,6 +217,9 @@
 #if TARGET_HOST_BLACKBERRY
 #include "blackberry/fg_internal_blackberry.h"
 #endif
+#if TARGET_HOST_OGC
+#include "ogc/fg_internal_ogc.h"
+#endif
 
 
 /* -- GLOBAL TYPE DEFINITIONS ---------------------------------------------- */
@@ -444,20 +454,22 @@ struct tagSFG_Timer
 typedef struct tagSFG_Context SFG_Context;
 struct tagSFG_Context
 {
-    SFG_WindowHandleType  Handle;    /* The window's handle                 */
+    SFG_WindowHandleType Handle;     /* The window's handle                 */
     SFG_WindowContextType Context;   /* The window's OpenGL/WGL context     */
+    SFG_WindowColormapType cmap;     /* The window's colormap/palette */
+	int cmap_size;                   /* colormap number of entries */
 
     SFG_PlatformContext pContext;    /* The window's FBConfig (X11) or device context (Windows) */
 
-    int             DoubleBuffered;  /* Treat the window as double-buffered */
+    int DoubleBuffered;              /* Treat the window as double-buffered */
 
     /* When drawing geometry to vertex attribute buffers, user specifies
      * the attribute indices for vertices, normals and/or texture coords
      * to freeglut. Those are stored here
      */
-    GLint           attribute_v_coord;
-    GLint           attribute_v_normal;
-    GLint           attribute_v_texture;
+    GLint attribute_v_coord;
+    GLint attribute_v_normal;
+    GLint attribute_v_texture;
 };
 
 
@@ -910,6 +922,11 @@ struct tagSFG_StrokeFont
  */
 #define _JS_MAX_BUTTONS 32
 
+/* Platforms can override this in their internal include file */
+#ifndef MAX_NUM_JOYSTICKS
+#define MAX_NUM_JOYSTICKS 2
+#endif
+
 #if TARGET_HOST_MACINTOSH
 #    define _JS_MAX_AXES  9
 typedef struct tagSFG_PlatformJoystick SFG_PlatformJoystick;
@@ -1063,10 +1080,11 @@ SFG_Menu*   fgCreateMenu( FGCBMenuUC menuCallback, FGCBUserData userData );
 void        fgDestroyMenu( SFG_Menu* menu );
 
 /* Joystick device management functions, defined in fg_joystick.c */
-int         fgJoystickDetect( void );
-void        fgInitialiseJoysticks( void );
-void        fgJoystickClose( void );
-void        fgJoystickPollWindow( SFG_Window* window );
+int fgJoystickDetect(void);
+void fgInitialiseJoysticks(void);
+void fgJoystickClose(void);
+void fgJoystickPollWindow(SFG_Window *window);
+void fgJoystickRawRead(SFG_Joystick *joy, int *buttons, float *axes);
 
 /* InputDevice Initialisation and Closure */
 int         fgInputDeviceDetect( void );
@@ -1166,7 +1184,7 @@ SFG_Proc fgPlatformGetProcAddress( const char *procName );
 #define ATTRIB_VAL(a,v) {ATTRIB(a); ATTRIB(v);}
 
 int fghMapBit( int mask, int from, int to );
-int fghIsLegacyContextRequested( void );
+int fghIsLegacyContextRequested( SFG_Window *win );
 void fghContextCreationError( void );
 int fghNumberOfAuxBuffersRequested( void );
 
